#ifndef SRC_MYGEOMETRY_HH
#define SRC_MYGEOMETRY_HH

#include <dune/common/fvector.hh>

#include <dune/fufem/arithmetic.hh>

#ifdef HAVE_CAIROMM
#include <cairomm/context.h>
#include <cairomm/fontface.h>
#include <cairomm/surface.h>
#endif

namespace MyGeometry {
namespace {
  using LocalVector = Dune::FieldVector<double, 2>;
  using LocalMatrix = Dune::FieldMatrix<double, 2, 2>;

  LocalVector createVector(double x, double y) {
    LocalVector ret{ 0 };
    ret[0] = x;
    ret[1] = y;
    return ret;
  }

  LocalMatrix createMatrix(double a11, double a12, double a21, double a22) {
    LocalMatrix ret{ 0 };
    ret[0][0] = a11;
    ret[0][1] = a12;
    ret[1][0] = a21;
    ret[1][1] = a22;
    return ret;
  }

  LocalVector convexCombination(LocalVector const &x, LocalVector const &y) {
    LocalVector ret{ 0 };
    Arithmetic::addProduct(ret, 0.5, x);
    Arithmetic::addProduct(ret, 0.5, y);
    return ret;
  }
}

namespace reference {
  double const rightLeg = 0.27;
  double const leftLeg = 1.00;
  double const leftAngle = atan(rightLeg / leftLeg);
  double const viscoHeight = 0.06; // Height of the viscous bottom layer
  double const weakLen = 0.20;     // Length of the weak zone

  LocalVector const A = createVector(0, 0);
  LocalVector const B = createVector(leftLeg, -rightLeg);
  LocalVector const C = createVector(leftLeg, 0);

  LocalVector const Z = createVector(0.35, 0);
  LocalVector const Y = createVector(0.35, -0.35 / leftLeg * rightLeg);
  LocalVector const X = createVector(Y[0] - weakLen * std::cos(leftAngle),
                                     Y[1] + weakLen * std::sin(leftAngle));

  LocalVector const U = createVector(X[0], 0);

  LocalVector const K =
      createVector(B[0] - leftLeg * viscoHeight / rightLeg, B[1] + viscoHeight);
  LocalVector const M = createVector(B[0], B[1] + viscoHeight);

  LocalVector const G = convexCombination(A, X);
  LocalVector const H = convexCombination(X, Y);
  LocalVector const J = convexCombination(Y, B);

  LocalVector const I = createVector(Y[0] + G[0], Y[1] + G[1]);

  LocalVector const zenith = createVector(0, 1);
  LocalVector const rotatedZenith = createVector(-1, 0);

  LocalMatrix const rotation =
      createMatrix(std::cos(leftAngle), -std::sin(leftAngle),
                   std::sin(leftAngle), std::cos(leftAngle));
}

namespace {
  LocalVector rotate(LocalVector const &x) {
    LocalVector ret{ 0 };
    reference::rotation.mv(x, ret);
    return ret;
  }
}

LocalVector const A = rotate(reference::A);
LocalVector const B = rotate(reference::B);
LocalVector const C = rotate(reference::C);
LocalVector const G = rotate(reference::G);
LocalVector const H = rotate(reference::H);
LocalVector const I = rotate(reference::I);
LocalVector const J = rotate(reference::J);
LocalVector const K = rotate(reference::K);
LocalVector const M = rotate(reference::M);
LocalVector const U = rotate(reference::U);
LocalVector const X = rotate(reference::X);
LocalVector const Y = rotate(reference::Y);
LocalVector const Z = rotate(reference::Z);

LocalVector const zenith = rotate(reference::zenith);
LocalVector const rotatedZenith =
    rotate(reference::rotatedZenith); // FIXME: misnomer

double verticalProjection(LocalVector const &);
double horizontalProjection(LocalVector const &);

void write();

void render();
}
#endif
